@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018 Massachusetts Institute of Technology
+    2017, 2018, 2019 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -111,10 +111,10 @@ USA.
 	    (loop (cdr items) items))))))
 
 (define (with-finalizer-lock finalizer thunk)
-  (with-thread-mutex-lock
-      (gc-finalizer-mutex finalizer)
-    (lambda ()
-      (without-interruption thunk))))
+  (without-interruption
+   (lambda ()
+     (with-thread-mutex-lock (gc-finalizer-mutex finalizer)
+       thunk))))
 
 (define (with-gc-finalizer-lock finalizer thunk)
   (guarantee-gc-finalizer finalizer 'with-gc-finalizer-lock)
